'''
Created on 23.07.2012

@author: Jan Brohl <janbrohl@t-online.de>
@license: Simplified BSD License see scoutnet/license.txt
@copyright: Copyright (c) 2012, Jan Brohl <janbrohl@t-online.de>. All rights reserved.
'''
from pyscoutnet.rest import ScoutnetREST
import simplerest.util


class ScoutnetObject(object):
    _slots = tuple()
    _methods = tuple()

    def __init__(self, object_dict=dict(), api=ScoutnetREST()):
        self._api = api
        for k, v in object_dict.items():
            if k in self._slots:
                setattr(self, k, v)

    def __getattr__(self, name):
        if name in self._methods:
            return lambda query = None, *args: self._api.query(self.path() + name + "/", query, *args)
        else:
            raise AttributeError(name)

    def object_dict(self):
        return dict((k, getattr(self, k, None)) for k in self._slots)

    def path(self):
        return "%s/%i/" % (self.kind, self.id)

    def __repr__(self):
        return "%s(%r,%r)" % (self.__class__.__name__, self.object_dict(), self._api)

    def __str__(self):
        return "%s(%s)" % (self.__class__.__name__, self.object_dict())


class ScoutnetCollection(ScoutnetObject):
    _slots = ("kind", "element_kind", "elements")

    def __init__(self, object_dict, api=ScoutnetREST()):
        if isinstance(object_dict["elements"], dict):
            object_dict["elements"] = object_dict["elements"].items()
        ScoutnetObject.__init__(self, object_dict, api)

    def __getitem__(self, key):
        return self.elements[key]

    def object_dict(self):
        d = ScoutnetObject.object_dict(self)
        d["elements"] = tuple(self)
        return d


class ScoutnetEvent(ScoutnetObject):
    _slots = tuple("""
    description end_date start_date group_id id keywords kind
    last_modified_by last_modified_on location organizer start_date start_time target_group
    target_group title uid url url_text zip
    """.split())
    _methods = tuple("""
    group
    """.split())


class ScoutnetGroup(ScoutnetObject):
    _slots = tuple("""
    city country district federal_state global_id internal_id kind layer name zip
    """.split())
    _methods = tuple("""
    events parent parents sections urls children
    """.split())

    @property
    def id(self):
        return self.global_id


class ScoutnetSection(ScoutnetObject):
    _slots = tuple("""
    color end_age id kind start_age
    """.split())


class ScoutnetUrl(ScoutnetObject):
    _slots = tuple("""
    group_id id kind text url
    """.split())


class ScoutnetAPI(ScoutnetObject):
    _methods = tuple("""
    event events group groups section sections url urls
    """.split())

    def path(self):
        return ""


class ObjectHook(simplerest.util.ObjectHook):
    kind2class = dict(section=ScoutnetSection, url=ScoutnetUrl,
                      group=ScoutnetGroup, event=ScoutnetEvent,
                      collection=ScoutnetCollection)

    def __init__(self, api):
        self.api = api
        simplerest.util.ObjectHook.__init__(self)

    def __call__(self, obj):
        d = simplerest.util.ObjectHook.__call__(self, obj)
        cls = self.kind2class.get(obj.get("kind", None), None)
        if cls:
            return cls(d, api=self.api)
        else:
            return d


def API():
    oh = ObjectHook(None)
    api = ScoutnetREST(object_hook=oh)
    oh.api = api
    return ScoutnetAPI(api=api)
