# -*- coding: utf-8 -*-
'''
Created on 22.06.2012

@author: Jan Brohl <janbrohl@t-online.de>
@license: Simplified BSD License see license.txt
@copyright: Copyright (c) 2012, Jan Brohl <janbrohl@t-online.de>. All rights reserved.
'''
from pyscoutnet.rest import ScoutnetAPI
from datetime import datetime, timedelta
from simplerest.util import TALVirtualPath
from numbers import Integral


class Lazy(TALVirtualPath):
    """
    @todo: dokumentieren
    @todo: sauberer implementieren
    @todo: benutze neue api-version
    """
    _objects = frozenset(["group", "section", "event"])
    _attributes = frozenset(["parents", "parent"])
    _collections = frozenset(["groups", "sections", "events"])
    _cache_dict = dict()
    _caching = 5.0

    def __init__(self, path=[], basepath=[], api=ScoutnetAPI()):
        TALVirtualPath.__init__(self, path=path, basepath=basepath, api=api)

    def __getattr__(self, name):
        return self[name]

    def _pathsplit(self):
        path = self.path_as_list
        i = len(path) - 1
        while i >= 0:
            pi = path[i]
            if pi in self._collections or pi in self._attributes:
                if len(path) > i + 1 and isinstance(path[i + 1], Integral):
                    return path[:i] + [pi.rstrip("s")], path[i + 2:]
                return path[:i + 1], path[i + 1:]
            elif pi in self._objects:
                return path[:i + 2], path[i + 2:]
            i -= 1
        return path, []

    @staticmethod
    def _find(obj, subpath):
        for k in subpath:
            try:
                obj = getattr(obj, k)
            except (AttributeError, TypeError):
                try:
                    obj = obj[int(k)]
                except:
                    obj = obj[k]
        return obj

    def __call__(self, query=None, *args):
        path, subpath = self._pathsplit()
        api = self.kwarg("api")
        if path:
            pfad = "/".join(str(p) for p in path) + "/"
        else:
            pfad = ""
        if query is None and self._caching:
            if (pfad not in self._cache_dict
                or
                (datetime.now() - self._cache_dict[pfad][1])
                >
                timedelta(seconds=self._caching)):
                #print "cache miss"
                obj = api.get(pfad)
                if obj["kind"] == "exception":
                    raise ScoutnetException(obj["message"], pfad)
                self._cache_dict[pfad] = (obj, datetime.now())
            else:
                #print "cache hit"
                obj = self._cache_dict[pfad][0]
        else:
            obj = api.query(pfad, query, *args)
        return self._find(obj, subpath)


class ScoutnetException(Exception):
    pass
