# -*- coding: utf-8 -*-
'''
Created on 27.06.2012

@warning: almost untested

@author: Jan Brohl <janbrohl@t-online.de>
@license: Simplified BSD License see license.txt
@copyright: Copyright (c) 2012, Jan Brohl <janbrohl@t-online.de>. All rights reserved.
'''
from wsgiref.simple_server import make_server
from wsgiref.util import request_uri
from json import dumps
from urlparse import urlsplit, parse_qs
from urllib import urlencode, unquote, quote
import datetime

umlauts_map = {u"ä": u"ae", u"ö": u"oe", "ü": u"ue", u"ß": u"ss"}


def deUmlaut(u):
    get = umlauts_map.get
    return "".join(get(c, c) for c in u)


class App(object):
    direct_map = dict(dpsg=["groups", "3"], wosm=["group", "657"], vcp=["group", "1273"], bdp=["group", "2371"], rdp=["group", "2195"],
                      termin=["event"], termine=["events"], ereignis=["event"], ereignisse=["events"],
                      einheit=["group"], einheiten=["groups"]
                      )

    def __init__(self, api):
        self.api = api

    def map_path(self, path):
        #apply direct path mapping
        dmg = self.direct_map.get
        urllist = []
        for u in (deUmlaut(unquote(i).lower()) for i in unicode(path).split("/") if i):
            urllist.extend(dmg(u, [u]))
        #correct numerus
        for i in range(len(urllist)):
            if urllist[i].endswith("s") and i + 1 < len(urllist):
                urllist[i] = urllist[i][:-1]

        return "/".join(quote(u, "") for u in urllist) + "/", dict()

    def map(self, path, query):
        select = query.pop("select", [])
        q = query.pop("pfadiql", None)
        if q:
            a = query["arg"]
        elif query:
            it = query.items()
            q = " and ".join(("%s == ?"if len(v) == 1 else "%s in ?") % k for k, v in it)
            a = list(((v if len(v) > 1 else v[0]) if v else ["", None]) for k, v in it)
        else:
            q = None
            a = []
        return path, select, q, a

    def __call__(self, environ, start_response):
        method = environ["REQUEST_METHOD"]
        parsed = urlsplit(request_uri(environ))
        url = environ["PATH_INFO"].strip("/")
        query = parse_qs(parsed.query, keep_blank_values=True)
        url, q2 = self.map_path(url)
        if q2 and query:
            return self.response_obj(start_response, "500 Internal Server Error", {"kind": "exception", "message": "Dieser Pfad untestützt noch keine Querys: %r" % url})
        path, select, pfadi_ql, args = self.map(url, query)
        if method == "GET":
            obj = self.api.query(path, pfadi_ql, *args)
            return self.response_obj(start_response, "200 OK", obj, select)
        else:
            return self.response_obj(start_response, "501 Not Implemented", {"kind": "exception", "message": "Methode \"%s\" wird (noch) nicht unterstützt." % method})

    def response_obj(self, start_response, code, obj, select=None):
        if obj.get("kind", None) == "collection":
            try:
                obj["elements"] = list(obj["elements"].values())
            except:
                pass
            if select:
                obj = list(dict(kv for kv in i.items()if kv[0] in select) for i in obj)
        else:
            if select:
                obj = dict(kv for kv in obj.items()if kv[0] in select)
        try:
            ret = dumps(obj, default=default, indent=1)
        except:
            return self.response_obj(start_response, "500 Internal Server Error", {"kind": "exception", "message": "Konnte Objekt nicht serialisieren: %r" % obj})
        headers = [("Content-Type", "application/json; charset=UTF-8"), ("content-length", str(len(ret)))]
        start_response(code, headers)
        return [ret]

    def make_server(self, host="localhost", port=80):
        return make_server(host, port, self)

def default(obj):
    if isinstance(obj, (datetime.date, datetime.datetime, datetime.time)):
        return str(obj)
    else:
        raise TypeError
